/*
 * Copyright 1990 Eric Ng
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * without any warranty whatsoever, without even the implied warranties
 * of merchantability or fitness for a particular purpose.  See the
 * accompanying GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to:
 *
 * Free Software Foundation, Inc.
 * 675 Massachusetts Avenue
 * Cambridge, Massachusetts 02139
 */

/* 0.00.00 16-Feb-90 Created [erc]
 * 0.01.00 19-Feb-90 Added UNIX compatibility
 *		     Added validity test
 *		     Added support for different operating systems
 *		     Fixed switch character for UNIX systems [erc]
 */
#define PURLOIN_C

#include <stdio.h>

#if defined (BSD)			/* BSD UNIX 4.3 */
#include <sys/file.h>
#define DATATYPE    int 		/* 32-bit integer */
#define OPEN_FLAGS  O_RDWR		/* open() flags */
#endif /* BSD */

#if defined (MSDOS)			/* MS-DOS 3.3 */
#include <stdlib.h>
#include <fcntl.h>
#if !defined (__TURBOC__)
#include <sys/types.h>
#endif /* __TURBOC__ */
#include <sys/stat.h>
#include <io.h>
#define DATATYPE    long		/* 32-bit integer */
#define L_SET	    SEEK_SET		/* lseek() whence */
#define OPEN_FLAGS  O_RDWR | O_BINARY	/* open() flags */
#endif /* MSDOS */

#if defined (SUNOS)			/* SunOS 4.0.3 */
#include <fcntl.h>
#include <sys/file.h>
#define OPEN_FLAGS  O_RDWR		/* open() flags */
#define DATATYPE    int 		/* 32-bit integer */
#endif /* SUNOS */

#if defined (SYSV)			/* UNIX System V */
#include <fcntl.h>
#define L_SET	    0			/* lseek() whence */
#define OPEN_FLAGS  O_RDWR		/* open() flags */
#define DATATYPE    int 		/* 32-bit integer */
#endif /* SYSV */

#include "version.h"

#define CITY_LENGTH	17		/* length of city name */
#define FUNDS_OFFSET	3108		/* funds offset (in bytes) */

void error (char *msg, char *subst);
void rd (int d, char *buf, int nbytes);
void seek (int des, long offset);
void usage (void);

char *id = ID;				/* program id */
char *version = VERSION;		/* version number */
char *copyright = COPYRIGHT;		/* copyright notice */
char *build_date = __DATE__;		/* build date */

char *filename; 			/* current filename */

int
main (int argc, char **argv)
{
    unsigned char *buffer;		/* input/output buffer */
    char *name; 			/* city name */
    int arg;				/* argument ptr */
    int file_des;			/* file descriptor */
    register i; 			/* index */
    DATATYPE funds = 0; 		/* funds */
    DATATYPE old_funds; 		/* old funds */
    unsigned DATATYPE temp;		/* temporary variable */

    if (argc == 1)			/* check for arguments */
	usage ();

/* allocate the input/output buffers
 */
    buffer = (unsigned char *) malloc (sizeof (funds));
    name = (char *) malloc (CITY_LENGTH);
    if (buffer == NULL || name == NULL)
	error ("ran out of memory", NULL);

    arg = 1;
    while (arg < argc) {		/* iterate for every argument */

/* handle specified funds
 */
	if (argv[arg][0] == '-') {
	    funds = atol (&argv[arg++][1]);
	} /* if */

/* open the file (with error checking)
 */
	filename = argv[arg];
	file_des = open (filename, OPEN_FLAGS);
	if (file_des == -1)
	    error ("couldn't open file %s", filename);

/* check validity of data file
 */
	seek (file_des, (long) 0);
	rd (file_des, name, 2);
	if (name[0] != 0 || name[1] != 0x0d)
	    error ("not a valid city file %s", filename);

/* determine city name and amount of funds
 */
	rd (file_des, name, CITY_LENGTH);
	seek (file_des, (long) FUNDS_OFFSET);
	rd (file_des, (char *) buffer, sizeof (old_funds));

/* convert the SimCity-compatible 32-bit integer into something which
 * can be worked with
 */
	old_funds = 0;
	for (i = 0; i < sizeof (old_funds); i++)
	    old_funds = buffer[i] + (256 * old_funds);

/* display current amount of funds
 */
	printf ("%s (%s): $%ld", name, filename, (long) old_funds);

/* if necessary, replace with new funds
 */
	if (funds > 0 && funds != old_funds) {
	    seek (file_des, (long) FUNDS_OFFSET);

/* convert the new amount of funds into a SimCity-compatible 32-bit
 * integer
 */
	    temp = funds;
	    for (i = sizeof (funds); i > 0; i--) {
		buffer[i-1] = temp % 256;
		temp = temp / 256;
	    } /* for */

/* write to disk
 */
	    if (write (file_des, (char *) buffer, sizeof (funds))
		== -1) {
		    putchar ('\n');
		    error ("couldn't write file %s", filename);
	    } /* if */
	    printf (" to $%ld", (long) funds);
	} /* if */

/* close this file and reiterate for next argument
 */
	putchar ('\n');
	close (file_des);
	arg++;
    } /* while */

    free (buffer);			/* free the input/output buffer */
    exit (0);
} /* main() */

/* rd(d, buf, nbytes) calls the read() system call (with error checking)
 */
void
rd (int d, char *buf, int nbytes)
{
    if (read (d, buf, nbytes) != nbytes)
	error ("couldn't read file %s", filename);
} /* rd() */

/* seek(des, offset) calls the lseek() system call (with error checking)
 */
void
seek (int des, long offset)
{
    long file_pos;

    file_pos = lseek (des, offset, L_SET);
    if (file_pos != offset)
	error ("couldn't seek file %s", filename);
} /* seek() */

/* error(msg, subst) prints a message to the stderr stream and exits
 */
void
error (char *msg, char *subst)
{
    fprintf (stderr, "%s: ", id);
    fprintf (stderr, msg, subst);
    fputc ('\n', stderr);
    exit (-1);
} /* error() */

/* usage() prints a short usage note to the stderr stream and exits
 */
void
usage ()
{
    fprintf (stderr, "%s %s  %s  %s\n", id, version, copyright,
	build_date);
    fprintf (stderr, "Usage: %s [ -funds ] city ...\n", id);
    exit (-1);
} /* usage() */
